package com.example.civilizedcity.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.example.civilizedcity.common.BaseController;
import com.example.civilizedcity.common.Constants;
import com.example.civilizedcity.common.DateUtils;
import com.example.civilizedcity.common.ResponseBean;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

import com.example.civilizedcity.entity.SysUser;
import com.example.civilizedcity.entity.TaAttach;
import com.example.civilizedcity.entity.TaIssueProcess;
import com.example.civilizedcity.service.SysUserDutyService;
import com.example.civilizedcity.service.TaAttachService;
import com.example.civilizedcity.service.TaIssueProcessService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import io.swagger.models.auth.In;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.example.civilizedcity.entity.TaIssue;
import com.example.civilizedcity.service.TaIssueService;

/**
 * 问题表;(ta_issue)表控制层
 *
 * @author : http://njyunzhi.com
 * @date : 2022-12-12
 */
@Api(tags = "问题表对象功能接口")
@RestController
@RequestMapping("/")
public class TaIssueController extends BaseController {

    @Autowired
    private TaIssueService taIssueService;

    @Autowired
    private TaAttachService taAttachService;

    @Autowired
    private TaIssueProcessService taIssueProcessService;

    @Autowired
    private SysUserDutyService sysUserDutyService;

    /**
     * 通过ID查询单条数据
     *
     * @param issueId 主键
     * @return 实例对象
     */
    @ApiOperation("通过ID查询单条数据")
    @GetMapping("/taIssue/{id}")
    public ResponseBean queryById(@ApiParam("对象ID") @PathVariable Integer id) throws Exception {

        TaIssue taIssue = taIssueService.getById(id);
        List<TaAttach> attachList = taAttachService.getListBy(Constants.ATTACH_SOURCE_ISSUE, taIssue.getIssueId().toString());
        taIssue.setAttachList(attachList);

        return ResponseBean.success(taIssue);
    }

    /**
     * 分页查询
     *
     * @param pageNum  当前页码
     * @param pageSize 每页条数
     * @return 查询结果
     */
    @ApiOperation("分页查询")
    @GetMapping("/taIssue")
    public ResponseBean list(@ApiParam("页码") @RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum,
                             @ApiParam("单页数据量") @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize,
                             @ApiParam("问题单状态") @RequestParam(value = "bizStatus", required = false) String bizStatus,
                             @ApiParam("来源") @RequestParam(value = "sourceType", required = false) String sourceType) throws Exception {

        SysUser sysUser = currentUser();
        String orgId = Constants.ROOT_ORG.equals(sysUser.getOrgId()) ? null : sysUser.getOrgId();

        List<String> dutyList = sysUserDutyService.getListByUser(sysUser.getUserId());
        if (dutyList == null || dutyList.size() < 1) {
            return ResponseBean.error("暂无权限查询数据");
        }

        Boolean mine = false;
        Integer validateStatus = null;
        if (dutyList.contains(Constants.DUTY_INSPECTOR)) {
            mine = true;
        } else if (dutyList.contains(Constants.DUTY_MANAGER)) {
            mine = true;
            validateStatus = 0;
        } else {
            return ResponseBean.error("暂无权限查询数据");
        }


        IPage<TaIssue> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<TaIssue> queryWrapper = new QueryWrapper<>();

        if (Constants.PROCESS_START.equals(bizStatus)) {
            // 待交办
            queryWrapper.eq("process_node", bizStatus);
        } else if (Constants.PROCESS_ASSIGNED.equals(bizStatus)) {
            // 已交办
            queryWrapper.eq("process_node", bizStatus);
        } else if (Constants.PROCESS_END.equals(bizStatus)) {
            // 已办结
            queryWrapper.eq("process_node", bizStatus);
        } else if ("expired".equals(bizStatus)) {
            // 已逾期
            queryWrapper.ne("process_node", Constants.PROCESS_END);
            queryWrapper.le("expire_date", DateUtils.today());
        } else if ("reject".equals(bizStatus)) {
            // 已打回
            queryWrapper.ne("process_node", Constants.PROCESS_END);
            queryWrapper.eq("process_status", Constants.PROCESS_STATUS_REJECT);
        }

        if (mine && null != validateStatus) {
            Integer vst = validateStatus;
            queryWrapper.nested(q -> q.eq("create_user", sysUser.getUserId()).or().gt("validate_status", vst));
        } else if (mine) {
            queryWrapper.eq(mine, "create_user", sysUser.getUserId());
        } else if (null != validateStatus) {
            queryWrapper.gt(null != validateStatus, "validate_status", validateStatus);
        }

        queryWrapper.eq(Constants.ISSUE_SOURCE_FEEDBACK.equals(sourceType), "source_type", "feedback");
        queryWrapper.ne(Constants.ISSUE_SOURCE_INSPECTOR.equals(sourceType), "source_type", "feedback");
        queryWrapper.gt("status", Constants.STATUS_DELETE);
        queryWrapper.orderByDesc("create_date");
        IPage<TaIssue> result = taIssueService.page(pg, queryWrapper);

        return ResponseBean.success(result);
    }

    /**
     * 新增数据
     *
     * @param taIssue 实例对象
     * @return 实例对象
     */
    @ApiOperation("新增数据")
    @PostMapping("/taIssue")
    public ResponseBean add(@ApiParam("对象实体") @RequestBody TaIssue taIssue) throws Exception {
        SysUser user = currentUser();

        taIssue.setIssueId(null);
        taIssue.setOrgId(null); // 暂时没有责任单位
        taIssue.setProcessNode(Constants.PROCESS_START);
        taIssue.setProcessStatus(Constants.PROCESS_STATUS_READY);
        taIssue.setProcessNum(1);
        taIssue.setSourceType(Constants.ISSUE_SOURCE_INSPECTOR);
        taIssue.setStatus(Constants.STATUS_NORMAL);
        taIssue.setCreateUser(user.getUserId());
        taIssue.setUserName(user.getName());
        taIssue.setCreateDate(LocalDateTime.now());
        taIssueService.save(taIssue);

        // 保存文件
        List<TaAttach> attachList = taIssue.getAttachList();
        if (null != attachList && attachList.size() > 0) {
            for (TaAttach item : attachList) {
                item.setAttachId(null);
                item.setOwnerType(Constants.ATTACH_SOURCE_ISSUE);
                item.setOwnerId(taIssue.getIssueId().toString());
                item.setStatus(Constants.STATUS_NORMAL);
                item.setCreateDate(LocalDateTime.now());
            }
        }

        taAttachService.saveBatch(attachList);

        // 流程开始
        TaIssueProcess process = new TaIssueProcess();
        process.setIssueId(taIssue.getIssueId());
        process.setOrgId(user.getOrgId());
        process.setProcessNode(Constants.PROCESS_START);
        process.setProcessStatus(Constants.PROCESS_STATUS_READY);
        process.setStatus(Constants.STATUS_NORMAL);
        process.setCreateUser(user.getUserId());
        process.setUserName(user.getName());
        process.setCreateDate(LocalDateTime.now());
        taIssueProcessService.save(process);

        return ResponseBean.success(taIssue);
    }

    /**
     * 更新数据
     *
     * @param taIssue 实例对象
     * @return 实例对象
     */
    @ApiOperation("更新数据")
    @PutMapping("/taIssue/{id}")
    public ResponseBean edit(@ApiParam("对象实体") @RequestBody TaIssue taIssue,
                             @ApiParam("对象ID") @PathVariable Integer id) throws Exception {

        TaIssue origin = taIssueService.getById(id);
        if (null == origin || origin.getStatus() == Constants.STATUS_DELETE) {
            return ResponseBean.error("未找到原始问题单");
        }

        List<TaAttach> attachList = taIssue.getAttachList();
        if (null == attachList || attachList.size() == 0) {
            return ResponseBean.error("拍照或者视频没有上传");
        }

        // 清空负状态
        if (Constants.PROCESS_STATUS_REJECT.equals(origin.getProcessStatus())) {
            taIssue.setProcessStatus(Constants.PROCESS_STATUS_READY);
        }

        taIssue.setIssueId(id);
        taIssueService.updateById(taIssue);

        // 保存文件
        List<Integer> idList = new ArrayList<>();
        for (TaAttach item : attachList) {
            item.setOwnerType(Constants.ATTACH_SOURCE_ISSUE);
            item.setOwnerId(taIssue.getIssueId().toString());
            item.setStatus(Constants.STATUS_NORMAL);
            item.setCreateDate(LocalDateTime.now());

            if (null == item.getAttachId()) {
                taAttachService.save(item);
            } else {
                taAttachService.updateById(item);
            }
            idList.add(item.getAttachId());
        }

        // 删除部分文件
        taAttachService.deleteNotIn(idList, Constants.ATTACH_SOURCE_ISSUE, id.toString());

        return ResponseBean.success(taIssue);
    }

    /**
     * 通过主键删除数据
     *
     * @param issueId 主键
     * @return 是否成功
     */
    @SaCheckPermission("manager")
    @ApiOperation("通过主键删除数据")
    @DeleteMapping("/taIssue/{id}")
    public ResponseBean deleteById(@ApiParam("对象ID") @PathVariable Integer id) throws Exception {
//        taIssueService.removeLogicById(id);

        TaIssue taIssue = taIssueService.getById(id);
        if (null == taIssue || taIssue.getStatus() == Constants.STATUS_DELETE) {
            return ResponseBean.error("未找到问题");
        }

        taIssue.setUpdateUser(currentUser().getUserId());
        taIssue.setUpdateDate(LocalDateTime.now());
        taIssue.setStatus(Constants.STATUS_DELETE);

        taIssueService.updateById(taIssue);
        return ResponseBean.success("success");
    }
}