package com.example.civilizedcity.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.example.civilizedcity.common.BaseController;
import com.example.civilizedcity.common.ResponseBean;
import java.util.List;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.example.civilizedcity.entity.TaNotice;
import com.example.civilizedcity.service.TaNoticeService;

 /**
 * 通知公告;(ta_notice)表控制层
 * @author : http://njyunzhi.com
 * @date : 2022-12-12
 */
@Api(tags = "通知公告对象功能接口")
@RestController
@RequestMapping("/")
public class TaNoticeController extends BaseController {
    
    @Autowired
    private TaNoticeService taNoticeService;
    
    /** 
     * 通过ID查询单条数据 
     *
     * @param noticeId 主键
     * @return 实例对象
     */
    @ApiOperation("通过ID查询单条数据")
    @GetMapping("/taNotice/{id}")
    public ResponseBean queryById(@ApiParam("对象ID") @PathVariable String id) throws Exception {
        return ResponseBean.success(taNoticeService.getById(id));
    }
    
    /** 
     * 分页查询
     *
     * @param pageNum 当前页码
     * @param pageSize 每页条数
     * @return 查询结果
     */
    @ApiOperation("分页查询")
    @GetMapping("/taNotice")
    public ResponseBean list(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                            @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize) throws Exception {
        
        IPage<TaNotice> pg = new Page<>(pageNum, pageSize);
        // QueryWrapper<TaNotice> queryWrapper = new QueryWrapper<>();
        // queryWrapper.orderByDesc("create_date");
        IPage<TaNotice> result = taNoticeService.page(pg);
        
        return ResponseBean.success(result);
    }
    
    /** 
     * 新增数据
     *
     * @param taNotice 实例对象
     * @return 实例对象
     */
    @ApiOperation("新增数据")
    @PostMapping("/taNotice")
    public ResponseBean add(@ApiParam("对象实体") @RequestBody TaNotice taNotice) throws Exception {
        taNoticeService.save(taNotice);
        return ResponseBean.success(taNotice);
    }
    
    /** 
     * 更新数据
     *
     * @param taNotice 实例对象
     * @return 实例对象
     */
    @ApiOperation("更新数据")
    @PutMapping("/taNotice/{id}")
    public ResponseBean edit(@ApiParam("对象实体") @RequestBody TaNotice taNotice,
                            @ApiParam("对象ID") @PathVariable String id ) throws Exception {
        taNoticeService.updateById(taNotice);
        return ResponseBean.success(taNotice);
    }
    
    /** 
     * 通过主键删除数据
     *
     * @param noticeId 主键
     * @return 是否成功
     */
    @ApiOperation("通过主键删除数据")
    @DeleteMapping("/taNotice/{id}")
    public ResponseBean deleteById(@ApiParam("对象ID") @PathVariable String id){
        taNoticeService.removeLogicById(id);
        return ResponseBean.success("success");
    }
}