package com.example.civilizedcity.service.impl;

import com.example.civilizedcity.common.Constants;
import com.example.civilizedcity.common.EncryptUtils;
import com.example.civilizedcity.common.StringUtils;
import com.example.civilizedcity.entity.SysLogin;
import com.example.civilizedcity.entity.SysRole;
import com.example.civilizedcity.entity.SysUserRole;
import com.example.civilizedcity.mapper.SysLoginMapper;
import com.example.civilizedcity.mapper.SysUserDutyMapper;
import com.example.civilizedcity.mapper.SysUserRoleMapper;
import com.example.civilizedcity.vo.ChangePassword;
import com.example.civilizedcity.vo.LoginParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.example.civilizedcity.entity.SysUser;
import com.example.civilizedcity.mapper.SysUserMapper;
import com.example.civilizedcity.service.SysUserService;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

/**
 * 用户表;(sys_user)表服务实现类
 *
 * @author : http://www.chiner.pro
 * @date : 2022-12-12
 */
@Service
public class SysUserServiceImpl extends BaseServiceImpl<SysUserMapper, SysUser> implements SysUserService {

    @Autowired
    SysLoginMapper sysLoginMapper;

    @Autowired
    SysUserRoleMapper sysUserRoleMapper;

    @Autowired
    SysUserDutyMapper sysUserDutyMapper;

    @Override
    public SysUser login(LoginParam loginParm) throws Exception {
        SysLogin sysLogin = sysLoginMapper.getByAccount(loginParm.getAccount());
        if (null == sysLogin) {
            throw new Exception("用户名或者密码不正确");
        }

        String userId = sysLogin.getUserId();
        if (StringUtils.isEmpty(userId)) {
            throw new Exception("用户名或者密码不正确");
        }

        boolean isRight = checkPassword(loginParm.getPassword(), sysLogin.getPassword(), userId);
        if (!isRight) {
            throw new Exception("用户名或者密码不正确");
        }

        SysUser sysUser = getById(userId);
        if (Constants.STATUS_DELETE == sysUser.getStatus()) {
            throw new Exception("用户账户已删除");
        }

        if (Constants.STATUS_NORMAL != sysUser.getStatus()) {
            throw new Exception("用户已停用");
        }

        return sysUser;
    }

    @Override
    public boolean changePassword(SysUser user, ChangePassword param) throws Exception {
        String passwordEncrypted = EncryptUtils.md5(param.getOriginPassword(), user.getUserId());
        SysLogin sysLogin = sysLoginMapper.getByUserAndPassword(user.getUserId(), passwordEncrypted);

        if (null == sysLogin) {
            throw new Exception("原始密码不正确");
        }

        String newPassword = EncryptUtils.md5(param.getNewPassword(), user.getUserId());
        sysLogin.setPassword(newPassword);

        return sysLoginMapper.updateById(sysLogin) > 0;
    }

    @Override
    public boolean mergeData(SysUser sysUser) throws Exception {
        if (StringUtils.isEmpty(sysUser.getName())) {
            throw new Exception("用户名称不能为空");
        }

        LocalDateTime now = LocalDateTime.now();

        // 用户表
        String userId = sysUser.getUserId();
        if (StringUtils.isEmpty(userId)) {
            if (!StringUtils.isEmpty(sysUser.getPhone())) {
                SysUser origin = getExistBy("phone", sysUser.getPhone(), false, true);
                if (null != origin) {
                    throw new Exception("手机号重复");
                }
            }

            if (null == sysUser.getStatus()) {
                sysUser.setStatus(Constants.STATUS_NORMAL);
            }

            sysUser.setCreateDate(now);
            save(sysUser);
            userId = sysUser.getUserId();
        } else {
            if (!StringUtils.isEmpty(sysUser.getPhone())) {
                SysUser origin = getByButNot("phone", sysUser.getPhone(), "user_id", userId, true);
                if (null != origin) {
                    throw new Exception("手机号重复");
                }
            }

            //
            updateById(sysUser);
        }

        // 授权角色
        List<SysRole> rolesList = sysUser.getRolesList();
        if (null == rolesList || rolesList.size() == 0) {
            sysUserRoleMapper.deleteByMap(new HashMap<String, Object>() {{
                put("user_id", sysUser.getUserId());
            }});
        } else {
            List<Integer> idList = new ArrayList<>();
            for (SysRole role : rolesList) {
                SysUserRole item = new SysUserRole();
                item.setUserId(sysUser.getUserId());
                item.setRoleId(role.getRoleId());

                SysUserRole exists = sysUserRoleMapper.getByUserAndRole(sysUser.getUserId(), role.getRoleId());
                if (null != exists) {
                    item.setId(exists.getId());
                } else {
                    sysUserRoleMapper.insert(item);
                }

                idList.add(item.getId());
            }

            // 去除垃圾数据
            sysUserRoleMapper.deleteNotIn(sysUser.getUserId(), idList);
        }

        return true;
    }

    @Override
    public List<String> getRoleList(String userId) {
        return sysUserRoleMapper.getRoleList(userId);
    }

    @Override
    public List<String> getPermissionList(String userId) {
        return sysUserDutyMapper.getListByUser(userId);
    }

    boolean checkPassword(String src, String targ, String salt) {
        String newPass = EncryptUtils.md5(src, salt);
        return targ.equals(newPass);
    }
}